﻿using System;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

[Serializable]
public class HeatSource
{
    public HeatObject HeatObject;
    public float Weight;
}

public class HeatObject : MonoBehaviour {
    public List<HeatSource> HeatSources;
    public float InitialTemperature;
    public float MinTemperatureColoringLimit;
    public float MaxTemperatureColoringLimit;
    public Color MinTemperatureColor;
    public Color MaxTemperatureColor;
    public float CurrentTemperature;
    public bool IsTurnOffable;
    Text temperatureIndicator;
    Environment environment;
    MeshRenderer cubeRenderer = null;
    RoomManager manager = null;

	// Use this for initialization
	void Start () {
        temperatureIndicator = gameObject.GetComponentInChildren<Text>();
        environment = FindObjectOfType<Environment>();
        CurrentTemperature = InitialTemperature;
        manager = GetComponent<RoomManager>();
        if (manager != null)
            cubeRenderer = manager.RoomGeometry.GetComponent<MeshRenderer>();
	}

    public void Reset()
    {
        CurrentTemperature = InitialTemperature;
    }
	
	// Update is called once per frame
	void Update () {
        if (environment.IsTimeRunning && HeatSources.Count > 0)
        {
            TimeSpan timeDifference = environment.CurrentTime - environment.LastCurrentTime;
            float totalTemperature = 0f;
            float totalWeight = 0f;
            foreach (var source in HeatSources)
            {
                totalTemperature += source.HeatObject.CurrentTemperature * source.Weight;
                totalWeight += source.Weight;
            }
            CurrentTemperature +=
                (totalTemperature / totalWeight - CurrentTemperature) * ((float)(timeDifference.TotalSeconds / 3600) * totalWeight);
        }
        if (cubeRenderer != null)
        {
            float temperature = CurrentTemperature;
            if (temperature < MinTemperatureColoringLimit)
                temperature = MinTemperatureColoringLimit;
            else if (temperature > MaxTemperatureColoringLimit)
                temperature = MaxTemperatureColoringLimit;
            temperature = (temperature - MinTemperatureColoringLimit) / (MaxTemperatureColoringLimit - MinTemperatureColoringLimit);
            if (cubeRenderer != null)
                cubeRenderer.material.color = Color.Lerp(MinTemperatureColor, MaxTemperatureColor, temperature);
            if (temperatureIndicator != null)
                temperatureIndicator.text = string.Format("{0:0.0}", CurrentTemperature);
        }
        else
        {
            //cubeRenderer = manager.RoomGeometry.GetComponent<MeshRenderer>();
            //print("My cube-renderer is hiding! :D");
        }
	}
}
