﻿using System;
using System.Collections.Generic;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using UnityEngine;
using UnityEngine.Networking;
using UnityEngine.UI;

public class NetworkUtilities : MonoBehaviour {
    public string ServerIP;
    public int ServerPort;
    public float connectionRetryDelay = 5f;
    Thread listenerThread;
    float listenerThreadStartTime;
    Stack<string> receivedMessages;
    Stack<string> queuedMessages;
    TcpClient client = null;
    NetworkStream stream = null;
    NetworkClient[] networkItems;

    void Awake()
    {
        receivedMessages = new Stack<string>();
        queuedMessages = new Stack<string>();
        networkItems = (NetworkClient[])GameObject.FindObjectsOfType<NetworkClient>();
        listenerThreadStartTime = Time.unscaledTime - 10f;
    }

    void SendToServer(string message)
    {
        if (client == null || stream == null)
            return;
        try
        {
            byte[] buffer = System.Text.Encoding.UTF8.GetBytes(message);
            stream.Write(buffer, 0, buffer.Length);
        }
        catch (Exception ex)
        {
            print(ex);
        }
    }

    public void TestWebRequest()
    {
        //WebRequest webRequest = WebRequest.Create("http://localhost:55558/Equipment");
        //WebResponse webResponse = webRequest.GetResponse();
        //byte[] responseBytes = new byte[16384];
        //webResponse.GetResponseStream().Read(responseBytes, 0, (int)webResponse.ContentLength);
        //print(System.Text.Encoding.UTF8.GetString(responseBytes));
        UnityWebRequest webRequest = UnityWebRequest.Get("http://localhost:55558/Home/SomeJson");
        var webOperation = webRequest.SendWebRequest();
        webOperation.completed += new Action<AsyncOperation>(a => {
            print(webRequest.downloadHandler.text);
            });
    }

    void NetworkThread()
    {
        print("Listener thread started...");
        client = new TcpClient();
        try
        {
            client.Connect(ServerIP, ServerPort);
            stream = client.GetStream();
            byte[] receiveBuffer = new byte[1000];
            string message = string.Empty;
            int bytesRead = -1;
            while (bytesRead != 0)
            {
                bytesRead = stream.Read(receiveBuffer, 0, 1000);
                message = System.Text.Encoding.UTF8.GetString(receiveBuffer, 0, bytesRead);
                receivedMessages.Push(message);
            }
        }
        catch (Exception ex)
        {
            print(ex.Message);
        }
        client = null;
        print("Listener thread ended...");
    }
	
	// Update is called once per frame
	void Update () {
        if ((listenerThread == null || !listenerThread.IsAlive) && Time.unscaledTime - listenerThreadStartTime >= connectionRetryDelay)
        {
            listenerThread = new Thread(new ThreadStart(NetworkThread));
            listenerThreadStartTime = Time.unscaledTime;
            listenerThread.Start();
        }
        if (queuedMessages.Count > 0)
        {
            var message = queuedMessages.Pop();
            SendToServer(message);
        }
        if (receivedMessages.Count > 0)
        {
            var message = receivedMessages.Pop();
            print(message);
            foreach (var networkItem in networkItems)
            {
                if (message.StartsWith(networkItem.ID))
                {
                    if (networkItem.ClientType == NetworkClientType.Text)
                    {
                        Text text = networkItem.gameObject.GetComponent<Text>();
                        text.text = message.Substring(networkItem.ID.Length + 1);
                    }
                    if (networkItem.ClientType == NetworkClientType.Light)
                    {
                        Light light = networkItem.gameObject.GetComponent<Light>();
                        light.intensity = float.Parse(message.Substring(networkItem.ID.Length + 1));
                    }
                }
            }
        }
        if (Input.GetKeyDown(KeyCode.A)) {
            queuedMessages.Push("Testing...1...2...3...");
        }
        if (Input.GetKeyDown(KeyCode.S))
        {
            TestWebRequest();
        }
    }

    public void OnDestroy()
    {
        print("Destroying...");
        SendToServer("DISCONNECT");
    }
}
