﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class UIScript : MonoBehaviour {
    Environment environment;
    bool isPlaying = false;
    InputField inputTimeScale;
    InputField inputYear;
    InputField inputMonth;
    InputField inputDate;
    Button buttonPlayPause;

	// Use this for initialization
	void Start () {
        environment = FindObjectOfType<Environment>();
        InputField[] inputFields = FindObjectsOfType<InputField>();
        foreach (var inputField in inputFields)
        {
            switch (inputField.name)
            {
                case "InputTimeScale":
                    inputTimeScale = inputField;
                    break;
                case "InputYear":
                    inputYear = inputField;
                    break;
                case "InputMonth":
                    inputMonth = inputField;
                    break;
                case "InputDate":
                    inputDate = inputField;
                    break;
            }
        }
        Button[] buttons = FindObjectsOfType<Button>();
        foreach (var button in buttons)
        {
            if (button.name.Equals("ButtonPlayPause"))
                buttonPlayPause = button;
        }
        if (inputTimeScale == null ||
            inputYear == null ||
            inputMonth == null ||
            inputDate == null ||
            buttonPlayPause == null)
        {
            Debug.LogError("Couldn't find UI elements");
#if (UNITY_EDITOR)
            UnityEditor.EditorApplication.isPlaying = false;
#endif
        }
        inputYear.text = environment.CurrentTime.Year.ToString();
        inputMonth.text = environment.CurrentTime.Month.ToString();
        inputDate.text = environment.CurrentTime.Day.ToString();
        inputTimeScale.text = environment.TimeScale.ToString();
    }

    // Update is called once per frame
    void Update () {
        if (isPlaying)
            return;
        try
        {
        }
        catch (Exception)
        {
        }
    }

    public void DateUpdated()
    {
        try
        {
            environment.TimeScale = float.Parse(inputTimeScale.text);
            DateTime dt = new DateTime(
                int.Parse(inputYear.text),
                int.Parse(inputMonth.text),
                int.Parse(inputDate.text),
                environment.CurrentTime.Hour,
                environment.CurrentTime.Minute,
                environment.CurrentTime.Second
                );
            buttonPlayPause.interactable = true;
            environment.CurrentTime = dt;
            var heatObjects = FindObjectsOfType<HeatObject>();
            foreach (var heatObject in heatObjects)
                heatObject.Reset();
        }
        catch (Exception)
        {
            buttonPlayPause.interactable = false;
            print("Disabling button");
        }
    }

    public void PlayPause()
    {
        isPlaying = !isPlaying;
        if (isPlaying)
        {
            inputTimeScale.interactable =
                inputYear.interactable =
                inputMonth.interactable =
                inputDate.interactable = false;
            buttonPlayPause.GetComponentInChildren<Text>().text = "Pause";
            environment.TogglePlay();
        } else
        {
            inputTimeScale.interactable =
                inputYear.interactable =
                inputMonth.interactable =
                inputDate.interactable = true;
            buttonPlayPause.GetComponentInChildren<Text>().text = "Play";
            environment.TogglePlay();
        }
    }
}
